/*
 * FT801_funkcje.h
 *
 *	funkcje kontrolera graficznego FT80x
 *
 *  Created on: 25 gru 2015
 *      Author: Rysiek www.arskam.com
 */

#ifndef APPLICATION_PROCEDURY_FT801_FT801_FUNKCJE_H_
#define APPLICATION_PROCEDURY_FT801_FT801_FUNKCJE_H_

#include "stm32f4xx_hal.h"


void FT80x_F_DL_Start(unsigned int *p_cmdBufferRd, unsigned int *p_cmdBufferWr, unsigned int *p_cmdOffset);
void FT80x_F_DL_End(unsigned int *p_cmdOffset);
typedef struct S_CMD_TXT{
	int16_t x;			//poz.x tekstu
	int16_t y;			//poz.y tekstu
	int16_t font;		//font 0-31
	uint16_t options;	//opcje wywietlania
	const char* s;		//wskanik do wywietlanego tekstu
}s_cmd_txt;
void FT80x_F_CMD_Text(unsigned int *p_cmdOffset, s_cmd_txt cmd_txt);

typedef struct S_F_PICTURE{
	unsigned char 	*p_pict_buf;//wskanik do buforu danych obrazka
	uint16_t		siz_w;		//szerokoc obrazka w pikselach
	uint16_t		siz_h;		//wysokoc obrazka w liniach
	int16_t 		x;			//poz.x lewego grnego rogu obrazka
	int16_t 		y;			//poz.y lewego grnego roguobrazka
	uint8_t		pic_in_ram_g;	//0-brak danych obrazka w RAM_G, 1-dane obrazka w RAM_G
	uint8_t		color_a;		//0-max. przezroczystoc, 255-bez przezroczystoci
	uint32_t	adr_in_ram_g;	//adres buforu obrazka w RAM_G
}s_f_picture;
void FT80x_F_Picture(unsigned int *p_cmdOffset, s_f_picture f_picture);
void FT80x_F_Picture_v2(unsigned int *p_cmdOffset, s_f_picture f_picture);

typedef struct S_CMD_TOGGLE{
	int16_t x;			//poz.x przelcznika
	int16_t y;			//poz.y przelcznika
	int16_t w;			//szerokoc przelcznika
	int16_t font;		//numer fontu
	uint16_t options;	//opcje wygldu
	uint16_t state;		//stan ON/OFF
	const char* s;		//wskanik do wywietlanych napisw
}s_cmd_toggle;
void FT80x_F_CMD_Toggle(unsigned int *p_cmdOffset, s_cmd_toggle cmd_toggle);

void FT80x_F_DL_Color_RGB(unsigned int *p_cmdOffset, uint8_t r, uint8_t g, uint8_t b);

typedef struct S_CMD_BUTTON{
	int16_t 	x;					//poz.x lewy grny rg
	int16_t 	y;					//poz.y lewy grny rg
	uint16_t	siz_w;				//szerokoc obrazka w pikselach
	uint16_t	siz_h;				//wysokoc obrazka w liniach
	int16_t 	font;				//font 0-31
	uint16_t 	options;			//opcje wywietlania
	const char* s;					//wskanik do tekstu napisu
	uint8_t 	color_RGB_tab[3];	//kolor napisu
	uint32_t 	fgcolor;			//kolor elementu
	uint32_t 	gradcolor;			//gradient
}s_cmd_button;
void FT80x_F_CMD_Button(unsigned int *p_cmdOffset, s_cmd_button cmd_button);

typedef struct S_CMD_CLOCK{
	int16_t 	x;					//poz.x centrum
	int16_t 	y;					//poz.y centrum
	int16_t 	r;					//promie
	uint16_t 	options;			//opcje wywietlania
	uint16_t 	h;					//wywietlany czas: godziny
	uint16_t 	m;					//wywietlany czas: minuty
	uint16_t 	s;					//wywietlany czas: sekundy
	uint16_t 	ms;					//czas: milisekundy
	uint8_t 	color_RGB_tab[3];	//kolor wskanikw
	uint32_t 	bgcolor;			//kolor elementu
}s_cmd_clock;
void FT80x_F_CMD_Clock(unsigned int *p_cmdOffset, s_cmd_clock cmd_clock);

typedef struct S_CMD_GAUGE{
	int16_t 	x;					//poz.x centrum
	int16_t 	y;					//poz.y centrum
	int16_t 	r;					//promie
	uint16_t 	options;			//opcje wywietlania
	uint16_t 	maior;				//iloc podziaek zgrubnych    1-10
	uint16_t 	minor;				//iloc podziaek precyzyjnych 1-10
	uint16_t 	val;				//wskazywana wartoc od 0 do range
	uint16_t 	range;				//zakres maksymalny
	uint8_t 	color_RGB_tab[3];	//kolor wskanikw
	uint32_t 	bgcolor;			//kolor elementu
}s_cmd_gauge;
void FT80x_F_CMD_Gauge(unsigned int *p_cmdOffset, s_cmd_gauge cmd_gauge);

typedef struct S_CMD_DIAL{
	int16_t 	x;					//poz.x centrum
	int16_t 	y;					//poz.y centrum
	int16_t 	r;					//promie
	uint16_t 	options;			//opcje wywietlania
	uint16_t 	val;				//wskazywana wartoc od 0 do 65535 0x0000-d, 0x4000 -lewo, 0x8000 -gra, 0xc000 -prawo
	uint8_t 	color_RGB_tab[3];	//kolor wskanika
	uint32_t 	fgcolor;			//kolor elementu
}s_cmd_dial;
void FT80x_F_CMD_Dial(unsigned int *p_cmdOffset, s_cmd_dial cmd_dial);

typedef struct S_CMD_SLIDER{
	int16_t 	x;					//poz.x lewy grny rg
	int16_t 	y;					//poz.y lewy grny rg
	int16_t 	w;					//rozmiar poziomy w pikselach
	int16_t 	h;					//rozmiar pionowy w pikselach
	uint16_t 	options;			//opcje wywietlania
	uint16_t 	val;				//wskazywana wartoc od 0 do range
	uint16_t 	range;				//zakres maksymalny
	uint32_t 	fgcolor;			//kolor przycisku suwaka
	uint32_t 	bgcolor;			//kolor elementu
}s_cmd_slider;
void FT80x_F_CMD_Slider(unsigned int *p_cmdOffset, s_cmd_slider cmd_slider);

typedef struct S_CMD_SPINNER{
	int16_t 	x;					//poz.x centrum
	int16_t 	y;					//poz.y centrum
	uint16_t 	style;				//styl poczekajki
	uint16_t 	scale;				//wielkoc
	uint8_t 	color_RGB_tab[3];	//kolor wskanika
}s_cmd_spinner;
void FT80x_F_CMD_Spinner(unsigned int *p_cmdOffset, s_cmd_spinner cmd_spinner);

typedef struct S_F_POINT{
	int16_t 	x;					//poz.x centrum
	int16_t 	y;					//poz.y centrum
	uint16_t 	size;				//rozmiar w 1/16 piksela, wartoc minimum 16
	uint8_t 	color_RGB_tab[3];	//kolor
}s_f_point;
void FT80x_F_Point(unsigned int *p_cmdOffset, s_f_point f_point);

typedef struct S_F_LINE{
	int16_t 	x_start;			//poz.x pocztek
	int16_t 	y_start;			//poz.y pocztek
	int16_t 	x_stop;				//poz.x koniec
	int16_t 	y_stop;				//poz.y koniec
	uint16_t 	size;				//rozmiar w pikselach
	uint8_t 	color_RGB_tab[3];	//kolor
}s_f_line;
void FT80x_F_Line(unsigned int *p_cmdOffset, s_f_line f_line);

typedef struct S_F_RECTS{
	int16_t 	x1;					//poz.x wierzchoka
	int16_t 	y1;					//poz.y wierzchoka
	int16_t 	x2;					//poz.x przeciwlegego wierzchoka
	int16_t 	y2;					//poz.y przeciwlegego wierzchoka
	uint8_t 	color_RGB_tab[3];	//kolor
}s_f_rects;
void FT80x_F_Rects(unsigned int *p_cmdOffset, s_f_rects f_rects);

#endif /* APPLICATION_PROCEDURY_FT801_FT801_FUNKCJE_H_ */
