/*
 * FT801_funkcje.c
 *
 *	funkcje kontrolera graficznego FT80x
 *
 *  Created on: 25 gru 2015
 *      Author: Rysiek www.arskam.com
 */

/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx_hal.h"

/* Private variables ---------------------------------------------------------*/
SPI_HandleTypeDef hspi1;

// AN_312 FT800 header
#include "FT800.h"
#include "FT801_procedury.h"
#include "FT801_funkcje.h"

#include "Demo_Obrazy.h"

//-----------------------------------------------------------
//funkcja inicjacji listy wywietlacza kontrolera graficznego
//we: 	*cmdBufferRd -wskanik do rejestru pozycji odczytu z buforu koowego listy
//		*cmdBufferWr -wskanik do rejestru pozycji zapisu do buforu koowego listy
//		*cmdOffset	 -wskanik do rejestru przesunicia pozycji zapisu
void FT80x_F_DL_Start(unsigned int *p_cmdBufferRd, unsigned int *p_cmdBufferWr, unsigned int *p_cmdOffset)
{
unsigned int cmdBufferRd, cmdBufferWr, cmdOffset;

	cmdBufferRd =*p_cmdBufferRd;
	cmdBufferWr =*p_cmdBufferWr;
	cmdOffset =*p_cmdOffset;

	// Wait for graphics processor to complete executing the current command list
	// This happens when REG_CMD_READ matches REG_CMD_WRITE, indicating that all commands
	// have been executed.  The next commands get executed when REG_CMD_WRITE is updated again...
	// then REG_CMD_READ again catches up to REG_CMD_WRITE
	// This is a 4Kbyte ring buffer, so keep pointers within the 4K roll-over
	do
	{
		cmdBufferRd = ft800memRead16(REG_CMD_READ);					// Read the graphics processor read pointer
		cmdBufferWr = ft800memRead16(REG_CMD_WRITE); 				// Read the graphics processor write pointer
	}while (cmdBufferWr != cmdBufferRd);									// Wait until the two registers match

	cmdOffset = cmdBufferWr;															// The new starting point the first location after the last command

	ft800memWrite32(RAM_CMD + cmdOffset, (CMD_DLSTART));// Start the display list
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer

	ft800memWrite32(RAM_CMD + cmdOffset, (DL_CLEAR_RGB | BLACK));																											// Set the default clear color to black
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG));																											// Clear the screen - this and the previous prevent artifacts between lists																											// Attributes are the color, stencil and tag buffers
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer

	ft800memWrite32(RAM_CMD + cmdOffset, (DL_SAVE_CONTEXT));
	cmdOffset = incCMDOffset(cmdOffset, 4);

	*p_cmdBufferRd =cmdBufferRd;
	*p_cmdBufferWr =cmdBufferWr;
	*p_cmdOffset =cmdOffset;
}
//-----------------------------------------------------------
//funkcja wykonania instrukcji z listy wywietlacza kontrolera graficznego
//we:	*cmdOffset	 -wskanik do rejestru przesunicia pozycji zapisu
void FT80x_F_DL_End(unsigned int *p_cmdOffset)
{
unsigned int cmdOffset;

	cmdOffset =*p_cmdOffset;

	ft800memWrite32(RAM_CMD + cmdOffset, (DL_END));			// End the point
	cmdOffset = incCMDOffset(cmdOffset, 4);					// Update the command pointer

	ft800memWrite32(RAM_CMD + cmdOffset, (DL_DISPLAY));		// Instruct the graphics processor to show the list
	cmdOffset = incCMDOffset(cmdOffset, 4);					// Update the command pointer

	ft800memWrite32(RAM_CMD + cmdOffset, (CMD_SWAP));		// Make this list active
	cmdOffset = incCMDOffset(cmdOffset, 4);					// Update the command pointer


	ft800memWrite16(REG_CMD_WRITE, (cmdOffset));			// Update the ring buffer pointer so the graphics processor starts executing
	*p_cmdOffset =cmdOffset;
}
//-----------------------------------------------------------
//funkcja wywietlania tekstu
//we:	*cmdOffset	-wskanik do rejestru przesunicia pozycji zapisu
//		s_cmd_txt	-struktura zawierajca parametry funkcji
void FT80x_F_CMD_Text(unsigned int *p_cmdOffset, s_cmd_txt cmd_txt)
{
unsigned int cmdOffset;


	cmdOffset =*p_cmdOffset;

	ft800memWrite32(RAM_CMD + cmdOffset, CMD_TEXT);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	ft800memWrite16(RAM_CMD + cmdOffset, cmd_txt.x);
	cmdOffset = incCMDOffset(cmdOffset, 2);
	ft800memWrite16(RAM_CMD + cmdOffset, cmd_txt.y);
	cmdOffset = incCMDOffset(cmdOffset, 2);
	ft800memWrite16(RAM_CMD + cmdOffset, cmd_txt.font);
	cmdOffset = incCMDOffset(cmdOffset, 2);
	ft800memWrite16(RAM_CMD + cmdOffset, cmd_txt.options);
	cmdOffset = incCMDOffset(cmdOffset, 2);

	while (1)
	{
		ft800memWrite8(RAM_CMD + cmdOffset, *cmd_txt.s);
		cmdOffset = incCMDOffset(cmdOffset, 1);
		if (*cmd_txt.s =='\0') break;
		cmd_txt.s++;
	}
	while ((cmdOffset %4) !=0)
	{
		ft800memWrite8(RAM_CMD + cmdOffset, '\0');
		cmdOffset = incCMDOffset(cmdOffset, 1);
	}


	*p_cmdOffset =cmdOffset;

}
//-----------------------------------------------------------
//funkcja wywietlania obrazka w formacie RGB565
void FT80x_F_Picture(unsigned int *p_cmdOffset, s_f_picture f_picture)
{
int x, ile;
unsigned int cmdOffset;

	cmdOffset =*p_cmdOffset;

	//przepisanie danych obrazka do pamici RAM_G (Main graphics RAM) od adresu 0x00
	ile =(f_picture.siz_h *f_picture.siz_w) *2;
	for (x=0; x<ile; x++)
	{
		ft800memWrite8(RAM_G+x, *(f_picture.p_pict_buf+x));
	}
	//BITMAP_HANDLE
	ft800memWrite32(RAM_CMD + cmdOffset, DL_BITMAP_HANDLE | 0x00);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//BITMAP_SOURCE
	ft800memWrite32(RAM_CMD + cmdOffset, DL_BITMAP_SOURCE | 0x00);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//BITMAP_LAYOUT
	ft800memWrite32(RAM_CMD + cmdOffset, DL_BITMAP_LAYOUT | (RGB565<<19) | ((f_picture.siz_w*2)<<9) | f_picture.siz_h);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//BITMAP_SIZE
	ft800memWrite32(RAM_CMD + cmdOffset, DL_BITMAP_SIZE | (BILINEAR<<20) | (BORDER<<19) | (BORDER<<18) |
			(f_picture.siz_w<<9) | f_picture.siz_h);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//cmd_loadidentity()
	ft800memWrite32(RAM_CMD + cmdOffset, CMD_LOADIDENTITY);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//cmd_scale(65536, 65536);
	ft800memWrite32(RAM_CMD + cmdOffset, CMD_SCALE);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	ft800memWrite32(RAM_CMD + cmdOffset, 65536);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	ft800memWrite32(RAM_CMD + cmdOffset, 65536);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//cmd_setmatrix()
	ft800memWrite32(RAM_CMD + cmdOffset, CMD_SETMATRIX);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//BEGIN(BITMAPS)
	ft800memWrite32(RAM_CMD + cmdOffset, DL_BEGIN | BITMAPS);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//VERTEX2F(1/16 pix, 1/16 lin);
	ft800memWrite32(RAM_CMD + cmdOffset, DL_VERTEX2F | ((f_picture.x*16)<<15) | (f_picture.y*16));
	cmdOffset = incCMDOffset(cmdOffset, 4);

	*p_cmdOffset =cmdOffset;
}
//-----------------------------------------------------------
//funkcja wywietlania obrazka w formacie RGB565 wersja 2
//z obsug kanau A i kontrol adowania danych do pamici RAM_G
void FT80x_F_Picture_v2(unsigned int *p_cmdOffset, s_f_picture f_picture)
{
int x, ile;
unsigned int cmdOffset;

	cmdOffset =*p_cmdOffset;

	if (f_picture.pic_in_ram_g ==0)
	{
		//przepisanie danych obrazka do pamici RAM_G (Main graphics RAM) od adresu 0x00
		ile =(f_picture.siz_h *f_picture.siz_w) *2;
		for (x=0; x<ile; x++)
		{
			ft800memWrite8(RAM_G+x, *(f_picture.p_pict_buf+x));
		}
		f_picture.pic_in_ram_g =1;
	}
	//BITMAP_HANDLE
	ft800memWrite32(RAM_CMD + cmdOffset, DL_BITMAP_HANDLE | 0x00);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//BITMAP_SOURCE
	ft800memWrite32(RAM_CMD + cmdOffset, DL_BITMAP_SOURCE | f_picture.adr_in_ram_g);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//BITMAP_LAYOUT
	ft800memWrite32(RAM_CMD + cmdOffset, DL_BITMAP_LAYOUT | (RGB565<<19) | ((f_picture.siz_w*2)<<9) | f_picture.siz_h);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//BITMAP_SIZE
	ft800memWrite32(RAM_CMD + cmdOffset, DL_BITMAP_SIZE | (BILINEAR<<20) | (BORDER<<19) | (BORDER<<18) |
			(f_picture.siz_w<<9) | f_picture.siz_h);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//cmd_loadidentity()
	ft800memWrite32(RAM_CMD + cmdOffset, CMD_LOADIDENTITY);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//cmd_scale(65536, 65536);
	ft800memWrite32(RAM_CMD + cmdOffset, CMD_SCALE);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	ft800memWrite32(RAM_CMD + cmdOffset, 65536);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	ft800memWrite32(RAM_CMD + cmdOffset, 65536);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//cmd_setmatrix()
	ft800memWrite32(RAM_CMD + cmdOffset, CMD_SETMATRIX);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//BEGIN(BITMAPS)
	ft800memWrite32(RAM_CMD + cmdOffset, DL_BEGIN | BITMAPS);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//COLOR_A
	ft800memWrite32(RAM_CMD + cmdOffset, DL_COLOR_A | f_picture.color_a);
	cmdOffset = incCMDOffset(cmdOffset, 4);
	//VERTEX2F(1/16 pix, 1/16 lin);
	//ft800memWrite32(RAM_CMD + cmdOffset, DL_VERTEX2F | ((f_picture.x*16)<<15) | (f_picture.y*16));
	ft800memWrite32(RAM_CMD + cmdOffset, DL_VERTEX2II | ((f_picture.x)<<21) | (f_picture.y)<<12);
	cmdOffset = incCMDOffset(cmdOffset, 4);

	*p_cmdOffset =cmdOffset;
}

//-----------------------------------------------------------
//funkcja wywietlania elementu: przecznika
void FT80x_F_CMD_Toggle(unsigned int *p_cmdOffset, s_cmd_toggle cmd_toggle)
{
unsigned int cmdOffset;

		cmdOffset =*p_cmdOffset;

		ft800memWrite32(RAM_CMD + cmdOffset, CMD_BGCOLOR);
		cmdOffset = incCMDOffset(cmdOffset, 4);
		ft800memWrite32(RAM_CMD + cmdOffset, 0x002040);
		cmdOffset = incCMDOffset(cmdOffset, 4);

		ft800memWrite32(RAM_CMD + cmdOffset, CMD_TOGGLE);
		cmdOffset = incCMDOffset(cmdOffset, 4);
		ft800memWrite16(RAM_CMD + cmdOffset, cmd_toggle.x);
		cmdOffset = incCMDOffset(cmdOffset, 2);
		ft800memWrite16(RAM_CMD + cmdOffset, cmd_toggle.y);
		cmdOffset = incCMDOffset(cmdOffset, 2);
		ft800memWrite16(RAM_CMD + cmdOffset, cmd_toggle.w);
		cmdOffset = incCMDOffset(cmdOffset, 2);
		ft800memWrite16(RAM_CMD + cmdOffset, cmd_toggle.font);
		cmdOffset = incCMDOffset(cmdOffset, 2);
		ft800memWrite16(RAM_CMD + cmdOffset, cmd_toggle.options);
		cmdOffset = incCMDOffset(cmdOffset, 2);
		ft800memWrite16(RAM_CMD + cmdOffset, cmd_toggle.state);
		cmdOffset = incCMDOffset(cmdOffset, 2);
		while (1)
		{
			ft800memWrite8(RAM_CMD + cmdOffset, *cmd_toggle.s);
			cmdOffset = incCMDOffset(cmdOffset, 1);
			if (*cmd_toggle.s =='\0') break;
			cmd_toggle.s++;
		}
		while ((cmdOffset %4) !=0)
		{
			ft800memWrite8(RAM_CMD + cmdOffset, '\0');
			cmdOffset = incCMDOffset(cmdOffset, 1);
		}

		*p_cmdOffset =cmdOffset;

}
//-----------------------------------------------------------
//funkcja ustawiania koloru
void FT80x_F_DL_Color_RGB(unsigned int *p_cmdOffset, uint8_t r, uint8_t g, uint8_t b)
{
unsigned int cmdOffset;

			cmdOffset =*p_cmdOffset;

			ft800memWrite32(RAM_CMD + cmdOffset, DL_COLOR_RGB | (r<<16) | (g<<8) | b);
			cmdOffset = incCMDOffset(cmdOffset, 4);

			*p_cmdOffset =cmdOffset;

}
//-----------------------------------------------------------
//funkcja wywietlania elementu: przycisk
void FT80x_F_CMD_Button(unsigned int *p_cmdOffset, s_cmd_button cmd_button)
{
unsigned int cmdOffset;

			cmdOffset =*p_cmdOffset;

			FT80x_F_DL_Color_RGB(&cmdOffset, cmd_button.color_RGB_tab[0],
											 cmd_button.color_RGB_tab[1],
											 cmd_button.color_RGB_tab[2]);
			ft800memWrite32(RAM_CMD + cmdOffset, CMD_FGCOLOR);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, cmd_button.fgcolor);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, CMD_GRADCOLOR);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, cmd_button.gradcolor);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, CMD_BUTTON);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_button.x);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_button.y);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_button.siz_w);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_button.siz_h);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_button.font);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_button.options);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			while (1)
			{
				ft800memWrite8(RAM_CMD + cmdOffset, *cmd_button.s);
				cmdOffset = incCMDOffset(cmdOffset, 1);
				if (*cmd_button.s =='\0') break;
				cmd_button.s++;
			}
			while ((cmdOffset %4) !=0)
			{
				ft800memWrite8(RAM_CMD + cmdOffset, '\0');
				cmdOffset = incCMDOffset(cmdOffset, 1);
			}

			*p_cmdOffset =cmdOffset;
}
//-----------------------------------------------------------
//funkcja wywietlania elementu: zegar RTC
void FT80x_F_CMD_Clock(unsigned int *p_cmdOffset, s_cmd_clock cmd_clock)
{
unsigned int cmdOffset;

			cmdOffset =*p_cmdOffset;

			FT80x_F_DL_Color_RGB(&cmdOffset, cmd_clock.color_RGB_tab[0],
											 cmd_clock.color_RGB_tab[1],
											 cmd_clock.color_RGB_tab[2]);
			ft800memWrite32(RAM_CMD + cmdOffset, CMD_BGCOLOR);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, cmd_clock.bgcolor);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, CMD_CLOCK);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_clock.x);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_clock.y);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_clock.r);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_clock.options);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_clock.h);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_clock.m);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_clock.s);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_clock.ms);
			cmdOffset = incCMDOffset(cmdOffset, 2);

			*p_cmdOffset =cmdOffset;
}
//-----------------------------------------------------------
//funkcja wywietlania elementu: wskanik zegarowy
void FT80x_F_CMD_Gauge(unsigned int *p_cmdOffset, s_cmd_gauge cmd_gauge)
{
unsigned int cmdOffset;

			cmdOffset =*p_cmdOffset;

			FT80x_F_DL_Color_RGB(&cmdOffset, cmd_gauge.color_RGB_tab[0],
											 cmd_gauge.color_RGB_tab[1],
											 cmd_gauge.color_RGB_tab[2]);
			ft800memWrite32(RAM_CMD + cmdOffset, CMD_BGCOLOR);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, cmd_gauge.bgcolor);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, CMD_GAUGE);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_gauge.x);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_gauge.y);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_gauge.r);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_gauge.options);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_gauge.maior);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_gauge.minor);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_gauge.val);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_gauge.range);
			cmdOffset = incCMDOffset(cmdOffset, 2);

			*p_cmdOffset =cmdOffset;
}
//-----------------------------------------------------------
//funkcja wywietlania elementu: pokrto
void FT80x_F_CMD_Dial(unsigned int *p_cmdOffset, s_cmd_dial cmd_dial)
{
unsigned int cmdOffset;

			cmdOffset =*p_cmdOffset;

			FT80x_F_DL_Color_RGB(&cmdOffset, cmd_dial.color_RGB_tab[0],
											 cmd_dial.color_RGB_tab[1],
											 cmd_dial.color_RGB_tab[2]);
			ft800memWrite32(RAM_CMD + cmdOffset, CMD_FGCOLOR);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, cmd_dial.fgcolor);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, CMD_DIAL);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_dial.x);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_dial.y);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_dial.r);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_dial.options);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_dial.val);
			cmdOffset = incCMDOffset(cmdOffset, 2);

			ft800memWrite16(RAM_CMD + cmdOffset, 0);
			cmdOffset = incCMDOffset(cmdOffset, 2);

			*p_cmdOffset =cmdOffset;
}
//-----------------------------------------------------------
//funkcja wywietlania elementu: suwak
void FT80x_F_CMD_Slider(unsigned int *p_cmdOffset, s_cmd_slider cmd_slider)
{
unsigned int cmdOffset;

			cmdOffset =*p_cmdOffset;

			FT80x_F_DL_Color_RGB(&cmdOffset, 0, 56, 112);
			ft800memWrite32(RAM_CMD + cmdOffset, CMD_FGCOLOR);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, cmd_slider.fgcolor);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, CMD_BGCOLOR);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, cmd_slider.bgcolor);
			cmdOffset = incCMDOffset(cmdOffset, 4);

			ft800memWrite32(RAM_CMD + cmdOffset, CMD_SLIDER);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_slider.x);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_slider.y);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_slider.w);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_slider.h);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_slider.options);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_slider.val);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_slider.range);
			cmdOffset = incCMDOffset(cmdOffset, 2);

			ft800memWrite16(RAM_CMD + cmdOffset, 0);
			cmdOffset = incCMDOffset(cmdOffset, 2);

			*p_cmdOffset =cmdOffset;
}
//-----------------------------------------------------------
//funkcja wywietlania elementu: sygnalizator (poczekajka)
void FT80x_F_CMD_Spinner(unsigned int *p_cmdOffset, s_cmd_spinner cmd_spinner)
{
unsigned int cmdOffset;

			cmdOffset =*p_cmdOffset;

			FT80x_F_DL_Color_RGB(&cmdOffset, cmd_spinner.color_RGB_tab[0],
											 cmd_spinner.color_RGB_tab[1],
											 cmd_spinner.color_RGB_tab[2]);

			ft800memWrite32(RAM_CMD + cmdOffset, CMD_SPINNER);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_spinner.x);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_spinner.y);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_spinner.style);
			cmdOffset = incCMDOffset(cmdOffset, 2);
			ft800memWrite16(RAM_CMD + cmdOffset, cmd_spinner.scale);
			cmdOffset = incCMDOffset(cmdOffset, 2);

			*p_cmdOffset =cmdOffset;

}
//-----------------------------------------------------------
//funkcja wywietlania elementu: punkt
void FT80x_F_Point(unsigned int *p_cmdOffset, s_f_point f_point)
{
unsigned int cmdOffset;

			cmdOffset =*p_cmdOffset;

			FT80x_F_DL_Color_RGB(&cmdOffset, f_point.color_RGB_tab[0],
											 f_point.color_RGB_tab[1],
											 f_point.color_RGB_tab[2]);
			ft800memWrite32(RAM_CMD + cmdOffset, (DL_POINT_SIZE | f_point.size));
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, (DL_BEGIN | FTPOINTS));
			cmdOffset = incCMDOffset(cmdOffset, 4);
			//VERTEX2F(1/16 pix, 1/16 pix);
			ft800memWrite32(RAM_CMD + cmdOffset, DL_VERTEX2F | ((f_point.x *16)<<15) | (f_point.y *16)<<0);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, (DL_END));			// End the point
			cmdOffset = incCMDOffset(cmdOffset, 4);					// Update the command pointer

			*p_cmdOffset =cmdOffset;
}
//-----------------------------------------------------------
//funkcja wywietlania elementu: linia
void FT80x_F_Line(unsigned int *p_cmdOffset, s_f_line f_line)
{
unsigned int cmdOffset;

			cmdOffset =*p_cmdOffset;

			FT80x_F_DL_Color_RGB(&cmdOffset, f_line.color_RGB_tab[0],
											 f_line.color_RGB_tab[1],
											 f_line.color_RGB_tab[2]);
			//rozmiar w 1/16 piksela
			ft800memWrite32(RAM_CMD + cmdOffset, (DL_LINE_WIDTH | (f_line.size*16)));
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, (DL_BEGIN | LINES));
			cmdOffset = incCMDOffset(cmdOffset, 4);
			//VERTEX2F(1/16 pix, 1/16 pix);
			ft800memWrite32(RAM_CMD + cmdOffset, DL_VERTEX2F | ((f_line.x_start *16)<<15) | (f_line.y_start *16)<<0);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			//VERTEX2F(1/16 pix, 1/16 pix);
			ft800memWrite32(RAM_CMD + cmdOffset, DL_VERTEX2F | ((f_line.x_stop *16)<<15) | (f_line.y_stop *16)<<0);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, (DL_END));			// End the point
			cmdOffset = incCMDOffset(cmdOffset, 4);					// Update the command pointer

			*p_cmdOffset =cmdOffset;
}
//-----------------------------------------------------------
//funkcja wywietlania elementu: prostokt
void FT80x_F_Rects(unsigned int *p_cmdOffset, s_f_rects f_rects)
{
unsigned int cmdOffset;

			cmdOffset =*p_cmdOffset;

			FT80x_F_DL_Color_RGB(&cmdOffset, f_rects.color_RGB_tab[0],
											 f_rects.color_RGB_tab[1],
											 f_rects.color_RGB_tab[2]);
			//rozmiar w 1/16 piksela dla wierzchoka bez zaokrgle
			ft800memWrite32(RAM_CMD + cmdOffset, (DL_LINE_WIDTH | (1*16)));
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, (DL_BEGIN | RECTS));
			cmdOffset = incCMDOffset(cmdOffset, 4);
			//VERTEX2F(1/16 pix, 1/16 pix);
			ft800memWrite32(RAM_CMD + cmdOffset, DL_VERTEX2F | ((f_rects.x1 *16)<<15) | (f_rects.y1 *16)<<0);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			//VERTEX2F(1/16 pix, 1/16 pix);
			ft800memWrite32(RAM_CMD + cmdOffset, DL_VERTEX2F | ((f_rects.x2 *16)<<15) | (f_rects.y2 *16)<<0);
			cmdOffset = incCMDOffset(cmdOffset, 4);
			ft800memWrite32(RAM_CMD + cmdOffset, (DL_END));			// End the point
			cmdOffset = incCMDOffset(cmdOffset, 4);					// Update the command pointer

			*p_cmdOffset =cmdOffset;
}
